import os
from typing import List, Tuple

import investor8_sdk
from pandas import DataFrame, read_csv

from i8_terminal.config import SETTINGS_FOLDER
from i8_terminal.types.auto_complete_choice import AutoCompleteChoice


def sort_stocks(df: DataFrame) -> DataFrame:
    df["default_rank"] = 11
    default_rank = {
        "A": 1,
        "AAL": 2,
        "AAP": 3,
        "AAPL": 4,
        "AABV": 5,
        "ABC": 6,
        "ABMD": 7,
        "ABT": 8,
        "ACN": 9,
        "ADBE": 10,
    }
    df["default_rank"] = df["ticker"].apply(lambda x: default_rank.get(x, 11))
    df = df.sort_values("default_rank").reset_index(drop=True)
    return df[["ticker", "name"]]


def get_stocks() -> List[Tuple[str, str]]:
    companies_path = f"{SETTINGS_FOLDER}/companies.csv"
    if os.path.exists(companies_path):
        df = read_csv(companies_path)
    else:
        results = investor8_sdk.StockInfoApi().get_all_active_companies()
        df = DataFrame([d.to_dict() for d in results])[["ticker", "name"]]
        df = sort_stocks(df)
        df.to_csv(companies_path, index=False)

    return list(df.to_records(index=False))


class TickerParamType(AutoCompleteChoice):
    name = "ticker"

    def get_suggestions(self, keyword: str, pre_populate: bool = False) -> List[Tuple[str, str]]:
        if not self.is_loaded:
            self.set_choices(get_stocks())

        if pre_populate and keyword.strip() == "":
            return self._choices[: self.size]

        return self.search_keyword(keyword)

    def __repr__(self) -> str:
        return "TICKER"



import os
import sys

import click
import investor8_sdk
from click_repl import repl
from rich.console import Console

from i8_terminal.commands import cli
from i8_terminal.config import USER_SETTINGS
from i8_terminal.types.i8_auto_suggest import I8AutoSuggest
from i8_terminal.types.i8_completer import I8Completer


def init_commands() -> None:
    app_dir = os.path.join(os.path.join(os.path.dirname(sys.executable), "lib"), "i8_terminal")
    sys.path.append(app_dir)
    commands_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "commands")
    commands_dir = commands_dir if os.path.exists(commands_dir) else os.path.join(app_dir, "commands")
    ignore_dir = ["__pycache__"]
    for cmd in [p for p in os.listdir(commands_dir) if os.path.isdir(os.path.join(commands_dir, p))]:
        if cmd not in ignore_dir:
            for sub_cmd in os.listdir(os.path.join(commands_dir, cmd)):
                sub_cmd_splitted = sub_cmd.split(".")
                if (sub_cmd_splitted[-1] in ["py", "pyc"]) and sub_cmd_splitted[0] not in ["__init__"]:
                    __import__(f"i8_terminal.commands.{cmd}.{''.join(sub_cmd_splitted[:-1])}")

    @cli.command()
    def shell() -> None:
        console = Console()
        console.print("\n👋 Welcome to i8 Terminal!", style="yellow")
        console.print("Copyright © 2020-2022 Investoreight | https://www.i8terminal.io/\n")
        console.print("- Enter [magenta]?[/magenta] to get the list of commands.")
        console.print("- Enter [magenta]:q[/magenta] to exit the shell.\n")

        prompt_kwargs = {"completer": I8Completer(cli), "auto_suggest": I8AutoSuggest(cli)}
        repl(click.get_current_context(), prompt_kwargs=prompt_kwargs)

    @cli.command()
    def exit() -> None:
        os._exit(0)


def main() -> None:
    console = Console()
    with console.status("Starting Up...", spinner="material"):
        init_commands()

    if (not set(["user", "login"]).issubset(set(sys.argv[1:]))) and (
        not USER_SETTINGS.get("i8_core_api_key") or not USER_SETTINGS.get("i8_core_token")
    ):
        console.print(
            "Please login first to use i8 terminal using the following command:\n[magenta]i8 user login[/magenta]"
        )
        return

    investor8_sdk.ApiClient().configuration.api_key["apiKey"] = USER_SETTINGS.get("i8_core_api_key")
    investor8_sdk.ApiClient().configuration.api_key["Authorization"] = USER_SETTINGS.get("i8_core_token")
    investor8_sdk.ApiClient().configuration.api_key_prefix["Authorization"] = "Bearer"

    cli(obj={})


if __name__ == "__main__":
    main()
else:
    init_commands()



import codecs
import os
from setuptools import find_packages, setup
from typing import Any

def read(rel_path: str) -> str:
    """
    Read a file.
    """
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), "r") as fp:
        return fp.read()

# TODO: move get_version and its dependecies to a third module (maybe setup_utils.py)
def get_version(rel_path: str) -> Any:
    """
    Read version from a file.
    """
    for line in read(rel_path).splitlines():
        if line.startswith("__version__"):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")

PACKAGE_NAME = "i8_terminal"

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

# TODO: get this from requirments.txt?
requirments = ["click==8.0.3", "pandas==1.4.0", "requests==2.25.1", "arrow==1.0.3", "investor8-sdk==1.1.0", 
               "pyyaml==5.4.1", "rich==10.11.0", "plotly==5.3.1", "dash==2.0.0", "dash-bootstrap-components==1.0.0", 
               "kaleido==0.2.1", "openpyxl==3.0.9", "click-repl==0.2.0", "cx-Freeze==6.10"]
 
setup(
    name="i8-terminal",
    version=get_version(f"{PACKAGE_NAME}/__init__.py"),
    author="investoreight",
    author_email="info@investoreight.com",
    license="LICENSE.txt",
    description="Investor8 CLI",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    package_dir={PACKAGE_NAME: PACKAGE_NAME, "": PACKAGE_NAME},
    entry_points={"console_scripts": [f"i8={PACKAGE_NAME}.main:main"]},
    include_package_data=True,
    install_requires=requirments
)
